'use strict';

var meow = require('meow');
var updateNotifier = require('update-notifier');
var fs = require('fs');
var semver = require('semver');
var path = require('path');
var chalk = require('chalk');
var osTmpdir = require('os-tmpdir');
var lockfile = require('proper-lockfile');
var filenamify = require('filenamify');

var pkg = require('../package.json');
var run = require('./');

var notifier = updateNotifier({ pkg: pkg });
if (notifier.update) {
  notifier.notify();
  console.log('\n    ############################################\n    ############################################\n\n    ' + chalk.blue('An update is available.') + '\n\n    ' + chalk.yellow('Run "npm i -g borgjs" to update.') + '\n\n    ############################################\n    ############################################\n  ');
}

var cli = meow({
  help: '\n\n        Usage\n          $ borgjs <archive><options>\n\n          Options\n            -c, --config         config file path\n\n          Examples\n            # run borgjs\n            $ borgjs -c=/User/me/borgjs.config.js\n\n            #run borgjs specifying the archive name, and log output to a file\n            $ borgjs $(date +%Y-%m-%d-%H%M%S) -c /path/to/your/borgjs.config.js >> $(date +%Y-%m-%d-%H%M%S).log\n    ' }, {
  alias: {
    c: 'config',
    v: 'version'
  },
  default: {},
  string: ['config']
});

if (!cli.flags.config) {
  console.log(chalk.red('A valid configuration file must be passed.'));
  console.log(chalk.yellow('$ borgjs -c /path/to/your/borgjs.config.js'));
  console.log(chalk.green('See ' + chalk.bold('https://github.com/vesparny/borgjs#configuration') + ' for config options.'));
  process.exit(1);
}
var config = require(path.resolve(process.cwd(), cli.flags.config));
if (!config.repository) {
  console.log(chalk.red('The "repository" option must be specified in the config file'));
  process.exit(1);
}

var fileLock = path.join(osTmpdir(), filenamify(config.repository, { replacement: '-' }));
lockfile.lock(fileLock, function (err) {
  if (err) {
    console.log(chalk.red(err.message));
    console.log(chalk.cyan('\n      There is probably another borgjs runnign process.\n      You should either wait for it to finish or remove the following lock file.\n\n      ' + chalk.yellow(fileLock + '.lock') + '\n    '));
    process.exit(1);
  }
  if (!semver.satisfies(process.version, '>=4')) {
    console.log(chalk.red('Your node version must be >=4.'));
    process.exit(1);
  }
  if (!fs.existsSync(path.resolve(process.cwd(), cli.flags.config))) {
    console.log(chalk.red('The provided config file does not exist.'));
    process.exit(1);
  }
  run(config, cli.input[0]).then(function () {
    console.log(chalk.green('Backup succesfully executed.'));
    process.exit(0);
  }).catch(function (err) {
    console.log(chalk.red('An error has occurred: ', err));
    process.exit(1);
  });
});