'use strict';

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var execa = require('execa');
var fs = require('fs');
var chalk = require('chalk');
var utils = require('./utils');

module.exports = function setupCommands(config) {
  function runCommandAndWriteToFile(cmd, name) {
    utils.log(chalk.yellow('Executing command:') + '\nborg ' + name);
    cmd.stderr.pipe(process.stdout);
    return cmd.then(function (res) {
      var msg = 'Command run:\n' + res.cmd;
      utils.log(chalk.magenta(msg));
    });
  }

  function create() {
    if (!config.paths || config.paths.length === 0) {
      return Promise.reject(new Error('The "paths" option must be specified in the config file, and must contain at least one valid path.'));
    }
    var paths = config.paths.filter(fs.existsSync);
    if (!paths || paths.length === 0) {
      return Promise.reject(Error('No valid paths found in the "paths" option.'));
    }
    if (!config.repository) {
      return Promise.reject(new Error('The "repository" option must be specified in the config file.'));
    }

    var exclude = config.exclude ? config.exclude.reduce(function (pre, cur) {
      return pre.concat(['--exclude', cur]);
    }, []) : [];

    var compression = config.compression ? ['--compression', config.compression] : [];
    return runCommandAndWriteToFile(execa(config.borgPath || 'borg', ['create'].concat(compression, ['--stats', '-v', '--list', '--filter', 'AME?', '--checkpoint-interval', '120', config.repository + '::' + (config.archivePrefix ? config.archivePrefix + '-' : '') + config.archiveName], _toConsumableArray(exclude), _toConsumableArray(paths))), 'create');
  }

  function check() {
    if (!config.check) {
      return Promise.resolve();
    }
    return runCommandAndWriteToFile(execa(config.borgPath || 'borg', ['check', '-v', '--show-rc', config.repository]), 'check');
  }

  function prune() {
    if (!config.prune) {
      return Promise.resolve();
    }

    var prefix = config.archivePrefix ? ['--prefix', config.prefix] : [];
    var prune = config.prune ? Object.keys(config.prune).reduce(function (pre, cur) {
      return pre.concat([cur.length > 1 ? '--' + cur : '-' + cur, config.prune[cur]]);
    }, []) : [];
    return runCommandAndWriteToFile(execa(config.borgPath || 'borg', ['prune', '-v', '--show-rc ', '--list', config.repository].concat(prefix, _toConsumableArray(prune))), 'prune');
  }

  return {
    check: check,
    prune: prune,
    create: create
  };
};