'use strict';

var parallel = require('run-parallel');
var dateFormat = require('dateformat');

function run(config, archiveName) {
  if (!config) {
    return Promise.reject(new Error('A valid configuration object must be passed in.'));
  }
  var sendNativeNotification = require('./nativeNotifications');

  if (config.passphrase) {
    process.env.BORG_PASSPHRASE = config.passphrase;
  }
  config.archiveName = archiveName || dateFormat(new Date(), 'yyyy-mm-dd-HHMMss');
  var commands = require('./commands')(config);
  var sendMailNotification = require('./mailer')(config.email);
  var sendPushNotifications = require('./pushNotifications')(config.pushbullet);

  return commands.create().then(commands.check).then(commands.prune).then(function () {
    return new Promise(function (resolve) {
      var notifications = [function (cb) {
        return cb();
      }];
      var subject = '👌   [borgjs] backup succesfully executed';
      var message = 'Archive ' + config.archiveName + ' created.';
      var emailMessage = {
        subject: subject,
        body: '\n             <div>' + message + '</div>\n           '
      };
      config.sendSuccessMail && notifications.push(function (cb) {
        return sendMailNotification(emailMessage, cb);
      });
      config.pushbullet && notifications.push(function (cb) {
        return sendPushNotifications({ subject: subject, message: message }, cb);
      });
      config.sendSystemNotification && notifications.push(function (cb) {
        return sendNativeNotification({ subject: subject, message: message }, cb);
      });
      parallel(notifications, resolve);
    });
  }).catch(function (err) {
    return new Promise(function (resolve, reject) {
      var notifications = [function (cb) {
        return cb();
      }];
      var subject = '💣   [borgjs] error while executing backup';
      var message = 'Archive ' + config.archiveName + ' failed. Check logs.';
      var emailMessage = {
        subject: subject,
        body: '\n            <div>' + message + '</div>\n           '
      };
      config.sendErrorMail && notifications.push(function (cb) {
        return sendMailNotification(emailMessage, cb);
      });
      config.pushbullet && notifications.push(function (cb) {
        return sendPushNotifications({ subject: subject, message: message }, cb);
      });
      config.sendSystemNotification && notifications.push(function (cb) {
        return sendNativeNotification({ subject: subject, message: message }, cb);
      });
      parallel(notifications, reject.bind(null, err));
    });
  });
}

module.exports = run;